﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace GasolineTanker
{
    internal abstract class AbstractMap
    {
        private IDrawingObject _drawningObject = null;
        protected int[,] _map = null;
        protected int _width;
        protected int _height;
        protected float _size_x;
        protected float _size_y;
        protected readonly Random _random = new();
        protected readonly int _freeRoad = 0;
        protected readonly int _barrier = 1;

        private Direction GetOpositDirection(Direction d)
        {
            switch (d)
            {
                case Direction.None:
                    return Direction.None;
                case Direction.Up:
                    return Direction.Down;
                case Direction.Down:
                    return Direction.Up;
                case Direction.Left:
                    return Direction.Right;
                case Direction.Right:
                    return Direction.Left;
            }
            return Direction.None;
        }

        private int CheckCollision(float Left, float Right, float Top, float Bottom)
        {
            int starCoordinateX = (int)(Left / _size_x);
            int starCoordinateY = (int)(Right / _size_y);
            int endCoordinateX = (int)(Top / _size_x);
            int endCoordinateY = (int)(Bottom / _size_y);
            if (starCoordinateX < 0 || starCoordinateY < 0 || endCoordinateX >= _map.GetLength(1) || endCoordinateY >= _map.GetLength(0))
            {
                return 2;
            }
            for (int x = starCoordinateX; x <= endCoordinateX; x++)
            {
                for (int y = starCoordinateY; y <= endCoordinateY; y++)
                {
                    if (_map[x, y] == _barrier)
                    {
                        return 1;
                    }
                }
            }
            return 0;
        }

        public Bitmap CreateMap(int width, int height, IDrawingObject drawningObject)
        {
            _width = width;
            _height = height;
            _drawningObject = drawningObject;
            GenerateMap();
            while (!SetObjectOnMap())
            {
                GenerateMap();
            }
            return DrawMapWithObject();
        }
        
        public Bitmap MoveObject(Direction direction)
        {
            if (true)
            {
                _drawningObject.MoveObject(direction);
            }
            (float Left, float Right, float Top, float Bottom) = _drawningObject.GetCurrentPosition();
            if (CheckCollision(Left, Right, Top, Bottom) != 0)
            {
                _drawningObject.MoveObject(GetOpositDirection(direction));
            }
            return DrawMapWithObject();
        }
        private bool SetObjectOnMap()
        {
            if (_drawningObject == null || _map == null)
            {
                return false;
            }
            int x = _random.Next(0, 10);
            int y = _random.Next(0, 10);
            _drawningObject.SetObject(x, y, _width, _height);
            (float Left, float Right, float Top, float Bottom) = _drawningObject.GetCurrentPosition();
  
            while (CheckCollision(Left, Right, Top, Bottom) != 2)
            {
                int resoult;
                do
                {
                    resoult = CheckCollision(Left, Right, Top, Bottom);
                    if (resoult == 0)
                    {
                        _drawningObject.SetObject((int)Left, (int)Right, _width, _height);
                        return true;
                    }
                    else
                    {
                        Left += _size_x;
                    }
                } while (resoult != 2);
                Left = x;
                Right += _size_y;
            }
            return false;
        }
        private Bitmap DrawMapWithObject()
        {
            Bitmap bmp = new(_width, _height);
            if (_drawningObject == null || _map == null)
            {
                return bmp;
            }
            Graphics gr = Graphics.FromImage(bmp);
            for (int i = 0; i < _map.GetLength(0); ++i)
            {
                for (int j = 0; j < _map.GetLength(1); ++j)
                {
                    if (_map[i, j] == _freeRoad)
                    {
                        DrawRoadPart(gr, i, j);
                    }
                    else if (_map[i, j] == _barrier)
                    {
                        DrawBarrierPart(gr, i, j);
                    }
                }
            }
            _drawningObject.DrawningObject(gr);
            return bmp;
        }

        protected abstract void GenerateMap();
        protected abstract void DrawRoadPart(Graphics g, int i, int j);
        protected abstract void DrawBarrierPart(Graphics g, int i, int j);
    }
}

