﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace GasolineTanker
{
    public partial class FormGasolineTankerConfig : Form
    {
        DrawingGasolineTanker _gasolineTanker = null;
        private event Action<DrawingGasolineTanker> EventAddGasolineTanker;
        public FormGasolineTankerConfig()
        {
            InitializeComponent();
            panelBlack.MouseDown += PanelColor_MouseDown;
            panelPurple.MouseDown += PanelColor_MouseDown;
            panelGray.MouseDown += PanelColor_MouseDown;
            panelGreen.MouseDown += PanelColor_MouseDown;
            panelRed.MouseDown += PanelColor_MouseDown;
            panelWhite.MouseDown += PanelColor_MouseDown;
            panelYellow.MouseDown += PanelColor_MouseDown;
            panelBlue.MouseDown += PanelColor_MouseDown;

            buttonCancel.Click += (s, e) => Close();
        }

        public void AddEvent(Action<DrawingGasolineTanker> ev)
        {
            if (EventAddGasolineTanker == null)
            {
                EventAddGasolineTanker = ev;
            }
            else
            {
                EventAddGasolineTanker += ev;
            }
        }
        private void setDragEffect(DragEventArgs e, Type needTypeData, bool condition)
        {
            if (e.Data.GetDataPresent(needTypeData) && condition)
            {
                e.Effect = DragDropEffects.Copy;
            }
            else
            {
                e.Effect = DragDropEffects.None;
            }
        }

        private void LabelBaseColor_DragEnter(object sender, DragEventArgs e) => setDragEffect(e, typeof(Color), _gasolineTanker != null);
        private void LabelDopColor_DragEnter(object sender, DragEventArgs e) => setDragEffect(e, typeof(Color), _gasolineTanker != null && _gasolineTanker is DrawingImprovedGasolineTanker);
        private void LabelBaseColor_DragDrop(object sender, DragEventArgs e)
        {
            var color = (Color)e.Data.GetData(typeof(Color));
            if (_gasolineTanker is DrawingImprovedGasolineTanker)
            {
                _gasolineTanker = ((DrawingImprovedGasolineTanker)_gasolineTanker).Copy(bodyColor: color);
            }
            else if (_gasolineTanker is DrawingGasolineTanker)
            {
                _gasolineTanker = _gasolineTanker.Copy(bodyColor: color);
            }
            DrawGasolineTanker();
        }
        private void LabelDopColor_DragDrop(object sender, DragEventArgs e)
        {
            var color = (Color)e.Data.GetData(typeof(Color));
            var gasolineTanker = _gasolineTanker as DrawingImprovedGasolineTanker;
            if (gasolineTanker != null)
            {
                _gasolineTanker = gasolineTanker.Copy(dopColor: color);
            }
            DrawGasolineTanker();
        }
        private void ButtonOk_Click(object sender, EventArgs e)
        {
            EventAddGasolineTanker?.Invoke(_gasolineTanker);
            Close();
        }
        private void LabelObject_MouseDown(object sender, MouseEventArgs e)
        {
            (sender as Label).DoDragDrop((sender as Label).Name, DragDropEffects.Move | DragDropEffects.Copy);
        }
        private void PanelColor_MouseDown(object sender, MouseEventArgs e)
        {
            (sender as Control).DoDragDrop((sender as Control).BackColor, DragDropEffects.Move | DragDropEffects.Copy);
        }

        private void PanelObject_DragEnter(object sender, DragEventArgs e)
        {
            if (e.Data.GetDataPresent(DataFormats.Text))
            {
                e.Effect = DragDropEffects.Copy;
            }
            else
            {
                e.Effect = DragDropEffects.None;
            }
        }

        private void DrawGasolineTanker()
        {
            Bitmap bmp = new(pictureBoxObject.Width, pictureBoxObject.Height);
            Graphics gr = Graphics.FromImage(bmp);
            _gasolineTanker?.SetPosition(5, 5, pictureBoxObject.Width, pictureBoxObject.Height);
            _gasolineTanker?.DrawTransport(gr);
            pictureBoxObject.Image = bmp;
        }
        private void PanelObject_DragDrop(object sender, DragEventArgs e)
        {
            switch (e.Data.GetData(DataFormats.Text).ToString())
            {
                case "labelSimpleObject":
                    _gasolineTanker = new DrawingGasolineTanker((int)numericUpDownSpeed.Value, (int)numericUpDownWeight.Value, Color.White);
                    break;
                case "labelImrovedObject":
                    _gasolineTanker = new DrawingImprovedGasolineTanker((int)numericUpDownSpeed.Value, (int)numericUpDownWeight.Value, Color.White, Color.Black,
                        checkBoxBodyKit.Checked, checkBoxOrnamentWheel.Checked);
                    break;
            }
            DrawGasolineTanker();
        }
    }
}
