﻿using Microsoft.Extensions.Logging;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
namespace GasolineTanker
{
    public partial class FormMapWithSetGasolineTanker : Form
    {
        private readonly Dictionary<string, AbstractMap> _mapsDict = new()
        {
            { "Simple map", new SimpleMap() },
            { "Long map", new LongMap() }
        };
        private MapWithSetGasolienTankerGeneric<DrawingObjectGasolineTanker, AbstractMap> _mapGasolineTankerCollectionGeneric;
        private readonly MapsCollection _mapsCollection;
        private readonly ILogger _logger;
        public FormMapWithSetGasolineTanker(ILogger<FormMapWithSetGasolineTanker> logger)
        {
            InitializeComponent();
            _mapsCollection = new MapsCollection(pictureBox.Width, pictureBox.Height);
            _logger = logger;
            comboBoxSelectorMap.Items.Clear();
            foreach (var elem in _mapsDict)
            {
                comboBoxSelectorMap.Items.Add(elem.Key);
            }
        }
        private void ReloadMaps()
        {
            int index = listBoxMaps.SelectedIndex;
            listBoxMaps.Items.Clear();

            for (int i = 0; i < _mapsCollection.Keys.Count; i++)
            {
                listBoxMaps.Items.Add(_mapsCollection.Keys[i]);
            }

            if (listBoxMaps.Items.Count > 0 && (index == -1 || index >= listBoxMaps.Items.Count))
            {
                listBoxMaps.SelectedIndex = 0;
            }
            else if (listBoxMaps.Items.Count > 0 && index > -1 && index < listBoxMaps.Items.Count)
            {
                listBoxMaps.SelectedIndex = index;
            }
        }

        private void ButtonDeleteMap_Click(object sender, EventArgs e)
        {
            if (listBoxMaps.SelectedIndex == -1)
            {
                _logger.LogInformation($"Attempt to remove a non - existent card: {textBoxNewMapName.Text}");
                return;
            }
            if (MessageBox.Show($"Delete card {listBoxMaps.SelectedItem}?", "Removal", MessageBoxButtons.YesNo, MessageBoxIcon.Question) == DialogResult.Yes)
            {
                _mapsCollection.DelMap(listBoxMaps.SelectedItem?.ToString() ?? string.Empty);
                ReloadMaps();
                _logger.LogInformation($"Map removed: {listBoxMaps.SelectedItem?.ToString() ?? string.Empty}");
            }
        }
        private void ButtonAddGasolineTanker_Click(object sender, EventArgs e)
        {
            {
                var formGasolineTankerConfig = new FormGasolineTankerConfig();
                formGasolineTankerConfig.AddEvent(AddGasolineTankerOnMap);
                formGasolineTankerConfig.Show();
            }
        }
        private void AddGasolineTankerOnMap(DrawingGasolineTanker drawingGasolineTanker)
        {
            try
            {
                if (listBoxMaps.SelectedIndex == -1)
                {
                    _logger.LogInformation($"Attempt to add a feature to an unselected map");
                    return;
                }
                if (drawingGasolineTanker == null)
                {
                    MessageBox.Show("You need to select an object before adding");
                    _logger.LogInformation($"No object was selected to add to the map");
                    return;
                }
                DrawingObjectGasolineTanker gasolineTanker = new DrawingObjectGasolineTanker(drawingGasolineTanker);
                if (_mapsCollection[listBoxMaps.SelectedItem?.ToString() ?? string.Empty] + gasolineTanker != -1)
                {
                    MessageBox.Show("Object added");
                    pictureBox.Image = _mapsCollection[listBoxMaps.SelectedItem?.ToString() ?? string.Empty].ShowSet();
                    _logger.LogInformation($"Object added {drawingGasolineTanker} to map ");
                }
                else
                {
                    MessageBox.Show("Failed to add item");
                    _logger.LogInformation($"Failed to add item {drawingGasolineTanker} to map ");
                }
            }
            catch (StorageOverflowException ex)
            {
                _logger.LogWarning($"Storage full error: {ex.Message}");
                MessageBox.Show($"Storage full error: {ex.Message}", "Result", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void ButtonShowStorage_Click(object sender, EventArgs e)
        {
            if (listBoxMaps.SelectedIndex == -1)
            {
                return;
            }
            pictureBox.Image = _mapGasolineTankerCollectionGeneric.ShowSet();
        }

        private void ButtonShowOnMap_Click(object sender, EventArgs e)
        {
            if(listBoxMaps.SelectedIndex == -1)
            {
                return;
            }
            pictureBox.Image = _mapsCollection[listBoxMaps.SelectedItem?.ToString() ?? string.Empty].ShowOnMap();
        }
        private void ButtonMove_Click(object sender, EventArgs e)
        {
            if (listBoxMaps.SelectedIndex == -1)
            {
                return;
            }
            string name = ((Button)sender)?.Name ?? string.Empty;
            Direction dir = Direction.None;
            switch (name)
            {
                case "keyUp":
                    dir = Direction.Up;
                    break;
                case "keyDown":
                    dir = Direction.Down;
                    break;
                case "keyLeft":
                    dir = Direction.Left;
                    break;
                case "keyRight":
                    dir = Direction.Right;
                    break;
            }
            pictureBox.Image = _mapGasolineTankerCollectionGeneric.MoveObject(dir);
        }
        private void ComboBoxSelectorMap_SelectedIndexChanged(object sender, EventArgs e)
        {
            AbstractMap map = null;
            switch (comboBoxSelectorMap.Text)
            {
                case "Simple map":
                    map = new SimpleMap();
                    break;
                case "Long map":
                    map = new LongMap();
                    break;
            }
            if (map != null)
            {
                _mapGasolineTankerCollectionGeneric = new MapWithSetGasolienTankerGeneric<DrawingObjectGasolineTanker, AbstractMap>(
                    pictureBox.Width, pictureBox.Height, map);
            }
            else
            {
                _mapGasolineTankerCollectionGeneric = null;
            }
        }
        private void buttonAddMap_Click(object sender, EventArgs e)
        {
            if (comboBoxSelectorMap.SelectedIndex == -1 || string.IsNullOrEmpty(textBoxNewMapName.Text))
            {
                MessageBox.Show("Not all data is filled", "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                _logger.LogInformation("When adding card {0}", comboBoxSelectorMap.SelectedIndex == -1 ? "No card selected" : "No card named");
                return;
            }
            if (!_mapsDict.ContainsKey(comboBoxSelectorMap.Text))
            {
                MessageBox.Show("No such card", "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                _logger.LogWarning("No card named: {0}", textBoxNewMapName.Text);
                return;
            }
            _mapsCollection.AddMap(textBoxNewMapName.Text, _mapsDict[comboBoxSelectorMap.Text]);
            ReloadMaps();
            _logger.LogInformation("Added map {0}", textBoxNewMapName.Text); ;
        }
        private void buttonDeleteMap_Click(object sender, EventArgs e)
        {
            if (listBoxMaps.SelectedIndex == -1)
            {
                return;
            }
            if (MessageBox.Show($"Delete card {listBoxMaps.SelectedItem}?", "Удаление", MessageBoxButtons.YesNo, MessageBoxIcon.Question) == DialogResult.Yes)
            {
                _mapsCollection.DelMap(listBoxMaps.SelectedItem?.ToString() ?? string.Empty);
                _logger.LogInformation("Map removed {0}", listBoxMaps.SelectedItem?.ToString() ?? string.Empty);
                ReloadMaps();
            }
        }

        private void listBoxMaps_SelectedIndexChanged(object sender, EventArgs e)
        {
            pictureBox.Image = _mapsCollection[listBoxMaps.SelectedItem?.ToString() ?? string.Empty].ShowSet();
            _logger.LogInformation("You have navigated to the map named: {0}", listBoxMaps.SelectedItem?.ToString() ?? string.Empty);
        }

        private void buttonRemoveGasolineTanker_Click(object sender, EventArgs e)
        {
            if (listBoxMaps.SelectedIndex == -1)
            {
                return;
            }
            if (string.IsNullOrEmpty(maskedTextBoxPosition.Text))
            {
                return;
            }
            if (MessageBox.Show("Delete object?", "Removal", MessageBoxButtons.YesNo, MessageBoxIcon.Question) == DialogResult.No)
            {
                return;
            }
            int pos = Convert.ToInt32(maskedTextBoxPosition.Text);
            try
            {
                if (_mapsCollection[listBoxMaps.SelectedItem?.ToString() ?? string.Empty] - pos != null)
                {
                    MessageBox.Show("Object removed");
                    pictureBox.Image = _mapsCollection[listBoxMaps.SelectedItem?.ToString() ?? string.Empty].ShowSet();
                    _logger.LogInformation($"Object removed { pos}");
                }
                else
                {
                    MessageBox.Show("Failed to delete object");
                    _logger.LogInformation($"Failed to delete object {pos}");
                }
            }
            catch (GasolineTankerNotFoundException ex)
            {
                _logger.LogWarning($"Deletion error: {ex.Message}");
                MessageBox.Show($"Deletion error {ex.Message}");
            }
            catch (Exception ex)
            {
                _logger.LogWarning($"Unknown error: {ex.Message}");
                MessageBox.Show($"Unknown error {ex.Message}");
            }
        }

        private void SaveToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (saveFileDialog.ShowDialog() == DialogResult.OK)
            {
                try
                {
                    _mapsCollection.SaveData(saveFileDialog.FileName);
                    _logger.LogInformation("The save was successful. The file is located: {0}", saveFileDialog.FileName);
                    MessageBox.Show("The save was successful", "Result", MessageBoxButtons.OK, MessageBoxIcon.Information);
                }
                catch (Exception ex)
                {
                    MessageBox.Show($"Not preserved: {ex.Message}", "Result", MessageBoxButtons.OK, MessageBoxIcon.Error);
                    _logger.LogWarning("Failed to save file '{0}'. Error text: {1}", saveFileDialog.FileName, ex.Message);
                }
            }
        }

        private void LoadToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (openFileDialog.ShowDialog() == DialogResult.OK)
            {
                try
                {
                    _mapsCollection.LoadData(openFileDialog.FileName);
                    MessageBox.Show("Opening was successful", "Result", MessageBoxButtons.OK, MessageBoxIcon.Information);
                    _logger.LogInformation("Opening file '{0}' was successful", openFileDialog.FileName);
                    ReloadMaps();
                }
                catch (Exception ex)
                {
                    MessageBox.Show($"Failed to open: {ex.Message}", "Result", MessageBoxButtons.OK, MessageBoxIcon.Error);
                    _logger.LogWarning("Failed to open file {0}. Error text: {1}", openFileDialog.FileName, ex.Message);
                }
            }
        }
    }
}
